import sys
import re
import argparse
import shipyard_bp_utils as shipyard
from shipyard_s3 import S3Client
from shipyard_templates import ExitCodeException, ShipyardLogger, CloudStorage

from shipyard_s3.utils.exceptions import NoMatchesFound

logger = ShipyardLogger.get_logger()


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--bucket-name", dest="bucket_name", required=True)
    parser.add_argument(
        "--source-file-name-match-type",
        dest="source_file_name_match_type",
        choices={"exact_match", "regex_match"},
        required=True,
    )
    parser.add_argument(
        "--source-folder-name", dest="source_folder_name", default="", required=False
    )
    parser.add_argument("--source-file-name", dest="source_file_name", required=True)
    parser.add_argument(
        "--destination-file-name",
        dest="destination_file_name",
        default="",
        required=False,
    )
    parser.add_argument(
        "--destination-folder-name",
        dest="destination_folder_name",
        default="",
        required=False,
    )
    parser.add_argument("--s3-config", dest="s3_config", default=None, required=False)
    parser.add_argument("--aws-access-key-id", dest="aws_access_key_id", required=False)
    parser.add_argument(
        "--aws-secret-access-key", dest="aws_secret_access_key", required=False
    )
    parser.add_argument(
        "--aws-default-region", dest="aws_default_region", required=False
    )
    return parser.parse_args()


def main():
    try:
        args = get_args()
        bucket_name = args.bucket_name
        src_file = args.source_file_name
        src_folder = args.source_folder_name

        src_path = shipyard.files.combine_folder_and_file_name(src_folder, src_file)

        match_type = args.source_file_name_match_type
        target_dir = shipyard.files.clean_folder_name(args.destination_folder_name)

        if args.destination_folder_name:
            shipyard.files.create_folder_if_dne(target_dir)

        client = S3Client(
            aws_access_key=args.aws_access_key_id,
            aws_secret_access_key=args.aws_secret_access_key,
            region=args.aws_default_region,
        )
        client.connect()

        logger.info("Successfully connected to S3")

        if match_type == "regex_match":
            logger.info("Beginning to scan for file matches...")
            file_names = client.list_files(
                bucket_name=bucket_name,
                s3_folder=src_folder,
            )
            matching_file_names = shipyard.files.find_all_file_matches(
                file_names, re.compile(src_file)
            )
            if (n_matches := len(matching_file_names)) == 0:
                raise NoMatchesFound(src_file)

            logger.info(f" {n_matches} files found. Preparing to download...")

            for index, key_name in enumerate(matching_file_names, start=1):
                dest_name = shipyard.files.determine_destination_file_name(
                    source_full_path=key_name,
                    destination_file_name=args.destination_file_name,
                )

                dest_path = shipyard.files.determine_destination_full_path(
                    destination_folder_name=target_dir,
                    destination_file_name=dest_name,
                    source_full_path=key_name,
                    file_number=index if args.destination_file_name else None,
                )
                logger.info(f"Downloading file {index} of {len(matching_file_names)}")
                client.download(
                    bucket_name=bucket_name,
                    s3_path=key_name,
                    dest_path=dest_path,
                )
                logger.info(
                    f"Successfully downloaded s3://{bucket_name}/{key_name} to {dest_path}"
                )
        else:
            dest_path = shipyard.files.determine_destination_full_path(
                destination_folder_name=target_dir,
                destination_file_name=args.destination_file_name,
                source_full_path=src_path,
            )
            client.download(
                bucket_name=bucket_name,
                s3_path=src_path,
                dest_path=dest_path,
            )
            logger.info(f"Successfully downloaded s3://{src_path} to {dest_path}")

    except ExitCodeException as ec:
        logger.error(ec.message)
        sys.exit(ec.exit_code)
    except Exception as e:
        logger.error(f"Error in downloading from S3: {str(e)}")
        sys.exit(CloudStorage.EXIT_CODE_UNKNOWN_ERROR)


if __name__ == "__main__":
    main()



import re
import argparse
import sys
import shipyard_bp_utils as shipyard
from shipyard_s3 import S3Client
from shipyard_templates import CloudStorage, ShipyardLogger, ExitCodeException

from shipyard_s3.utils.exceptions import NoMatchesFound

logger = ShipyardLogger.get_logger()


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--source-file-name-match-type",
        dest="source_file_name_match_type",
        default="exact_match",
        choices={"exact_match", "regex_match"},
        required=False,
    )
    parser.add_argument("--source-file-name", dest="source_file_name", required=True)
    parser.add_argument(
        "--source-folder-name", dest="source_folder_name", default="", required=False
    )
    parser.add_argument(
        "--source-bucket-name", dest="source_bucket_name", default="", required=True
    )
    parser.add_argument(
        "--destination-folder-name",
        dest="destination_folder_name",
        default="",
        required=False,
    )
    parser.add_argument(
        "--destination-file-name",
        dest="destination_file_name",
        default=None,
        required=False,
    )
    parser.add_argument(
        "--destination-bucket-name",
        dest="destination_bucket_name",
        default="",
        required=True,
    )
    parser.add_argument("--aws-access-key-id", dest="aws_access_key_id", required=False)
    parser.add_argument(
        "--aws-secret-access-key", dest="aws_secret_access_key", required=False
    )
    parser.add_argument(
        "--aws-default-region", dest="aws_default_region", required=False
    )

    return parser.parse_args()


def main():
    try:
        args = get_args()
        src_file = args.source_file_name
        src_folder = (
            shipyard.files.clean_folder_name(args.source_folder_name)
            if args.source_folder_name
            else None
        )
        dest_folder = shipyard.files.clean_folder_name(args.destination_folder_name)
        match_type = args.source_file_name_match_type
        src_bucket = args.source_bucket_name
        dest_bucket = args.destination_bucket_name

        client = S3Client(
            aws_access_key=args.aws_access_key_id,
            aws_secret_access_key=args.aws_secret_access_key,
            region=args.aws_default_region,
        )

        logger.info("Successfully connected to S3")

        if match_type == "regex_match":
            logger.info("Beginning to scan for file matches...")
            file_names = client.list_files(bucket_name=src_bucket, s3_folder=src_folder)
            logger.debug(f"All file names: {file_names}")
            matching_file_names = shipyard.files.find_all_file_matches(
                file_names, re.compile(src_file)
            )
            logger.debug(f"matching file names: {matching_file_names}")
            if (n_matches := len(matching_file_names)) == 0:
                raise NoMatchesFound(src_file)

            logger.info(f" {n_matches} files found. Preparing to move...")

            for index, key_name in enumerate(matching_file_names, start=1):
                dest_path = shipyard.files.determine_destination_full_path(
                    destination_folder_name=dest_folder,
                    destination_file_name=args.destination_file_name,
                    source_full_path=key_name,
                    file_number=index if args.destination_file_name else None,
                )
                logger.info(f"Moving file {index} of {n_matches}")
                client.move(
                    src_bucket=src_bucket,
                    src_path=key_name,
                    dest_bucket=dest_bucket,
                    dest_path=dest_path,
                )

        else:
            if src_folder:
                src_path = shipyard.files.combine_folder_and_file_name(
                    src_folder, src_file
                )
            else:
                src_path = src_file

            dest_file = shipyard.files.determine_destination_file_name(
                source_full_path=src_path,
                destination_file_name=args.destination_file_name,
            )
            if args.destination_folder_name:
                dest_path = shipyard.files.determine_destination_full_path(
                    destination_folder_name=dest_folder,
                    destination_file_name=dest_file,
                    source_full_path=src_path,
                )
            else:
                dest_path = dest_file

            client.move(
                src_bucket=src_bucket,
                src_path=src_path,
                dest_bucket=dest_bucket,
                dest_path=dest_path,
            )
            logger.info(
                f"Successfully moved s3://{src_bucket}/{src_path} to s3://{dest_bucket}/{dest_path} "
            )
    except ExitCodeException as ec:
        logger.error(ec.message)
        sys.exit(ec.exit_code)
    except Exception as e:
        logger.error(
            f"An unhandled error occurred when attempting to move file: {str(e)}"
        )
        sys.exit(CloudStorage.EXIT_CODE_UNKNOWN_ERROR)


if __name__ == "__main__":
    main()



from typing import Optional, Dict, Any, List


def list_objects(
    s3_conn, bucket_name: str, prefix: str, continuation_token: Optional[str] = None
) -> Dict:
    """Helper function that returns the first 1000 of objects from a given bucket

    Args:
        s3_conn (): The S3 connection
        bucket_name: The name of  the bucket to scan
        prefix: The prefix (folder path) to use
        continuation_token: The refresh token

    Returns:

    """
    kwargs = {"Bucket": bucket_name, "Prefix": prefix}
    if continuation_token:
        kwargs["ContinuationToken"] = continuation_token

    response = s3_conn.list_objects_v2(**kwargs)
    return response


def get_files(response: dict) -> List[Any]:
    return [obj["Key"] for obj in response.get("Contents", [])]



import os
import re
import argparse
import ast
import sys

import shipyard_bp_utils as shipyard
import shipyard_templates

from shipyard_s3.s3 import S3Client
from shipyard_templates import ShipyardLogger, ExitCodeException

from shipyard_s3.utils.exceptions import NoMatchesFound

logger = ShipyardLogger.get_logger()


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--bucket-name", dest="bucket_name", required=True)
    parser.add_argument(
        "--source-file-name-match-type",
        dest="source_file_name_match_type",
        default="exact_match",
        choices={"exact_match", "regex_match"},
        required=False,
    )
    parser.add_argument("--source-file-name", dest="source_file_name", required=True)
    parser.add_argument(
        "--source-folder-name", dest="source_folder_name", default="", required=False
    )
    parser.add_argument(
        "--destination-folder-name",
        dest="destination_folder_name",
        default="",
        required=False,
    )
    parser.add_argument(
        "--destination-file-name",
        dest="destination_file_name",
        default=None,
        required=False,
    )
    parser.add_argument("--s3-config", dest="s3_config", default=None, required=False)
    parser.add_argument("--aws-access-key-id", dest="aws_access_key_id", required=False)
    parser.add_argument(
        "--aws-secret-access-key", dest="aws_secret_access_key", required=False
    )
    parser.add_argument(
        "--aws-default-region", dest="aws_default_region", required=False
    )
    parser.add_argument("--extra-args", dest="extra_args", required=False)
    return parser.parse_args()


def main():
    try:
        args = get_args()
        bucket_name = args.bucket_name
        source_file = args.source_file_name
        source_dir = args.source_folder_name if args.source_folder_name else os.getcwd()
        dest_file = (
            args.destination_file_name if args.destination_file_name else source_file
        )
        source_path = os.path.join(source_dir, source_file)
        match_type = args.source_file_name_match_type
        s3_config = args.s3_config
        extra_args = ast.literal_eval(args.extra_args if args.extra_args else "{}")

        client = S3Client(
            aws_access_key=args.aws_access_key_id,
            aws_secret_access_key=args.aws_secret_access_key,
            region=args.aws_default_region,
        )
        client.connect()

        logger.info("Successfully connected to S3")

        if match_type == "regex_match":
            logger.info("Beginning to scan for file matches...")
            file_names = shipyard.files.find_all_local_file_names(source_dir)
            matching_file_names = shipyard.files.find_all_file_matches(
                file_names, re.compile(source_file)
            )

            if (n_matches := len(matching_file_names)) == 0:
                raise NoMatchesFound(source_file)
            else:
                logger.info(f"{n_matches} files found. Preparing to upload...")

            for index, key_name in enumerate(matching_file_names, start=1):
                if args.destination_folder_name:
                    s3_folder = shipyard.files.clean_folder_name(
                        args.destination_folder_name
                    )
                    s3_path = shipyard.files.determine_destination_full_path(
                        destination_folder_name=s3_folder,
                        destination_file_name=args.destination_file_name,
                        source_full_path=key_name,
                        file_number=index,
                    )
                else:
                    s3_path = shipyard.files.determine_destination_file_name(
                        source_full_path=key_name,
                        destination_file_name=dest_file,
                        file_number=index,
                    )
                logger.info(f"Uploading file {index} of {len(matching_file_names)}")
                client.upload(
                    bucket_name=bucket_name,
                    source_file=key_name,
                    destination_path=s3_path,
                    extra_args=extra_args,
                )
                logger.info(
                    f"Successfully uploaded {key_name} to s3://{bucket_name}/{s3_path}"
                )
        else:
            if args.destination_folder_name:
                s3_folder = shipyard.files.clean_folder_name(
                    args.destination_folder_name
                )
                s3_path = shipyard.files.determine_destination_full_path(
                    destination_folder_name=s3_folder,
                    destination_file_name=dest_file,
                    source_full_path=source_path,
                )
            else:
                logger.debug("S3 folder not provided")
                s3_path = shipyard.files.determine_destination_file_name(
                    source_full_path=source_path, destination_file_name=dest_file
                )
                logger.debug(f"S3 path is {s3_path}")
            client.upload(
                bucket_name=bucket_name,
                source_file=source_path,
                destination_path=s3_path,
                extra_args=extra_args,
            )
            logger.info(
                f"Successfully loaded {source_path} to s3://{bucket_name}/{s3_path}"
            )
    except ExitCodeException as ue:
        logger.error(ue.message)
        sys.exit(ue.exit_code)
    except Exception as e:
        logger.error(
            f"An unhandled exception occurred when attempting to upload to S3: {str(e)}"
        )
        sys.exit(shipyard_templates.CloudStorage.EXIT_CODE_UNKNOWN_ERROR)


if __name__ == "__main__":
    main()



from shipyard_templates import ExitCodeException, CloudStorage
from typing import Optional

EXIT_CODE_MOVE_ERROR = 101
EXIT_CODE_REMOVE_ERR0R = 102
EXIT_CODE_DOWNLOAD_ERROR = 103
EXIT_CODE_INVALID_REGION = 104
EXIT_CODE_BUCKET_DNE = 105
EXIT_CODE_BUCKET_ACCESS = 106
EXIT_CODE_LIST_OBJECT_ERROR = 107


class InvalidCredentials(ExitCodeException):
    def __init__(self):
        self.message = "The provided credentials are incorrect, pleasure ensure you enter valid access key and secret"
        self.exit_code = CloudStorage.EXIT_CODE_INVALID_CREDENTIALS


class UploadError(ExitCodeException):
    def __init__(self, message: str):
        self.message = message
        self.exit_code = CloudStorage.EXIT_CODE_UPLOAD_ERROR


class MoveError(ExitCodeException):
    def __init__(self, message: str):
        self.message = message
        self.exit_code = EXIT_CODE_MOVE_ERROR


class RemoveError(ExitCodeException):
    def __init__(self, message: str):
        self.message = message
        self.exit_code = EXIT_CODE_REMOVE_ERR0R


class DownloadError(ExitCodeException):
    def __init__(self, message: str):
        self.message = message
        self.exit_code = EXIT_CODE_DOWNLOAD_ERROR


class NoMatchesFound(ExitCodeException):
    def __init__(self, regex: str):
        self.message = f"No matches were found for the regex `{regex}`"
        self.exit_code = CloudStorage.EXIT_CODE_FILE_MATCH_ERROR


class InvalidRegion(ExitCodeException):
    def __init__(self, region: str):
        self.message = f"The provided Access Key and Secret do not have access to the following region: {region}"
        self.exit_code = EXIT_CODE_INVALID_REGION


class InvalidBucketAccess(ExitCodeException):
    def __init__(self, bucket: str):
        self.message = f"The bucket `{bucket}` is not accesible to the provided Access Key and Secret"
        self.exit_code = EXIT_CODE_BUCKET_ACCESS


class BucketDoesNotExist(ExitCodeException):
    def __init__(self, bucket: str):
        self.message = f"The bucket `{bucket}` does not exist"
        self.exit_code = EXIT_CODE_BUCKET_DNE


class ListObjectsError(ExitCodeException):
    def __init__(self, bucket_name: str, s3_folder: Optional[str]):
        msg = f"Error in listing the contents of s3://{bucket_name}/"
        self.message = msg + s3_folder if s3_folder else msg
        self.exit_code = EXIT_CODE_LIST_OBJECT_ERROR



import os
import sys

from shipyard_templates import ShipyardLogger

from shipyard_s3 import S3Client

logger = ShipyardLogger.get_logger()


def main():
    sys.exit(
        S3Client(
            aws_access_key=os.getenv("AWS_ACCESS_KEY_ID"),
            aws_secret_access_key=os.getenv("AWS_SECRET_ACCESS_KEY"),
            region=os.getenv("AWS_DEFAULT_REGION"),
        ).connect()
    )


if __name__ == "__main__":
    main()



import os
from typing import Optional, Dict, Any, List

import boto3
from boto3.exceptions import S3UploadFailedError
from shipyard_templates import CloudStorage, ExitCodeException, ShipyardLogger

from shipyard_s3.utils import utils
from shipyard_s3.utils.exceptions import (
    BucketDoesNotExist,
    DownloadError,
    InvalidBucketAccess,
    InvalidCredentials,
    InvalidRegion,
    ListObjectsError,
    MoveError,
    RemoveError,
    UploadError,
)

logger = ShipyardLogger.get_logger()


class S3Client(CloudStorage):
    def __init__(
        self,
        aws_access_key: str,
        aws_secret_access_key: str,
        region: Optional[str] = None,
    ) -> None:
        self.aws_access_key = aws_access_key
        self.aws_secret_access_key = aws_secret_access_key
        self.region = region
        self._s3_conn = None

    @property
    def s3_conn(self):
        if not self._s3_conn:
            self._s3_conn = self.get_connection()
        return self._s3_conn

    def connect(self):
        try:
            boto3.client(
                "sts",
                region_name=self.region,
                aws_access_key_id=self.aws_access_key,
                aws_secret_access_key=self.aws_secret_access_key,
            ).get_caller_identity()
        except Exception as e:
            logger.authtest(
                f"Could not validate credentials to S3 with the provided credentials. Response from the "
                f"server: {str(e)}"
            )
            return 1
        else:
            logger.authtest("Successfully connected to S3")
            return 0

    def get_connection(self):
        if self._s3_conn:
            return self._s3_conn  # reuse existing connection
        try:
            logger.debug("Establishing connection with S3")
            client = boto3.client(
                "s3",
                region_name=self.region,
                aws_access_key_id=self.aws_access_key,
                aws_secret_access_key=self.aws_secret_access_key,
                config=None,
            )
        except Exception as e:
            logger.error(str(e))
            raise InvalidCredentials
        else:
            logger.debug(
                "Successfully established connection, initializing connection property"
            )
            return client

    def move(self, src_bucket: str, dest_bucket: str, src_path: str, dest_path: str):
        """Move or rename an object within an S3 bucket

        Args:
            src_bucket: The source bucket
            dest_bucket: The destination bucket (can be the same as the src_bucket) if you are not moving objects between buckets
            src_path: The path of where the source object is
            dest_path: The path to where the source objects should be moved to

        Raises:
            MoveError:
        """
        try:
            # check the access to the buckets
            self.check_bucket(src_bucket)
            self.check_bucket(dest_bucket)
            copy_source = {"Bucket": src_bucket, "Key": src_path}
            # move the object(s)
            self.s3_conn.copy_object(
                Bucket=dest_bucket, CopySource=copy_source, Key=dest_path
            )
            # delete the original
            self.s3_conn.delete_object(Bucket=src_bucket, Key=src_path)

        except (BucketDoesNotExist, InvalidBucketAccess):
            raise
        except Exception as e:
            if "IllegalLocationConstraintException" in str(e):
                logger.debug(f"Response from the server: {str(e)}")
                raise InvalidRegion(region=self.region)
            raise MoveError(f"Error in attempting to move file: {str(e)}")

    def remove(self, bucket_name: str, src_path: str):
        """Delete a file in an S3 bucket

        Args:
            bucket_name: The name of the bucket where the target file resides
            src_path: The full path (if nested in a folders) where the target file resides

        Raises:
            RemoveError:
        """
        try:
            folder = os.path.dirname(src_path)
            files = self.list_files(bucket_name=bucket_name, s3_folder=folder)
            if src_path not in files:
                raise ExitCodeException(
                    f"File {src_path} was not found. Ensure that the file name and folder name provided are correct.",
                    exit_code=CloudStorage.EXIT_CODE_FILE_NOT_FOUND,
                )
            # check the access to the bucket
            self.check_bucket(bucket_name)

            s3_response = self.s3_conn.delete_object(Bucket=bucket_name, Key=src_path)
        except (BucketDoesNotExist, InvalidBucketAccess, ListObjectsError):
            raise ExitCodeException(
                f"File {src_path} was not found. Ensure that the file name and folder name provided are correct.",
                exit_code=CloudStorage.EXIT_CODE_FILE_NOT_FOUND,
            )
        except ExitCodeException:
            raise
        except Exception as e:
            if "IllegalLocationConstraintException" in str(e):
                logger.debug(f"Response from the server: {str(e)}")
                raise InvalidRegion(region=self.region)
            raise RemoveError(message=str(e))
        else:
            logger.debug(f"Response from s3: {s3_response}")

    def upload(
        self,
        bucket_name: str,
        source_file: str,
        destination_path: Optional[str] = None,
        extra_args: Optional[Dict[Any, Any]] = None,
    ):
        """Upload a file to an S3 Bucket

        Args:
            bucket_name: The name of the bucket to load to
            source_file: The name or file path of the target file to load
            destination_path: The optional path of where the file should be loaded to
            extra_args: Additional optional configuration arguments to be passed

        Raises:
            UploadError
        """
        try:
            # check the access to the bucket
            self.check_bucket(bucket_name)
            s3_upload_config = boto3.s3.transfer.TransferConfig()
            s3_transfer = boto3.s3.transfer.S3Transfer(
                client=self.s3_conn, config=s3_upload_config
            )
            s3_transfer.upload_file(
                source_file, bucket_name, destination_path, extra_args=extra_args
            )
        except (BucketDoesNotExist, InvalidBucketAccess):
            raise
        except S3UploadFailedError as se:
            if "IllegalLocationConstraintException" in str(se):
                logger.debug(f"Response from the server: {str(se)}")
                raise InvalidRegion(region=self.region)
        except Exception as e:
            raise UploadError(f"Error in uploading to S3: {str(e)}")

    def download(self, bucket_name: str, s3_path: str, dest_path: str):
        """Download a file from an S3 bucket
        Args:
            bucket_name: The bucket to download from
            s3_path: The file path of the object(s) to fetch
            dest_path: The path to download the target file(s) to

        Raises:
            DownloadError:
        """
        try:
            # check the access to the bucket
            self.check_bucket(bucket_name)
            self.s3_conn.download_file(bucket_name, s3_path, dest_path)
        except (BucketDoesNotExist, InvalidBucketAccess):
            raise
        except Exception as e:
            raise DownloadError(
                f"Error in downloading s3 file to {dest_path}: {str(e)}"
            )

    def list_files(self, bucket_name: str, s3_folder: Optional[str]) -> List[str]:
        """Returns the list of all the files (objects) in in a bucket and folder

        Args:
            bucket_name: The bucket to scan
            s3_folder: The optional folder path to scan, if omitted the the root of the bucket will be used

        Returns:

        """
        try:
            response = utils.list_objects(
                s3_conn=self.s3_conn, bucket_name=bucket_name, prefix=s3_folder
            )
            logger.debug(
                f"Response from S3 when attempting to list objects: {response}"
            )
            file_names = utils.get_files(response)
            continuation_token = response.get("NextContinuationToken")

            while continuation_token:
                response = utils.list_objects(
                    s3_conn=self.s3_conn,
                    bucket_name=bucket_name,
                    prefix=s3_folder,
                    continuation_token=continuation_token,
                )
                file_names = file_names.append(utils.get_files(response))
                continuation_token = response.get("NextContinuationToken")
            return file_names
        except Exception as e:
            logger.debug(f"Response from S3 is {str(e)}")
            raise ListObjectsError(bucket_name, s3_folder)

    def check_bucket(self, bucket_name: str):
        try:
            response = self.s3_conn.list_objects_v2(Bucket=bucket_name)
            logger.debug(f"Able to access bucket {bucket_name}")
            # logger.debug(f"Contents of response: {response}")
        except self.s3_conn.exceptions.NoSuchBucket:
            raise BucketDoesNotExist(bucket_name)
        except Exception as e:
            raise InvalidBucketAccess(bucket_name)
